@@ -47,6 +47,36 @@ use lib::{cmp, fmt, hash, iter, mem, ops, ptr, slice};
 #[cfg(feature = "std")]
 use lib::io;
 
+// POINTER METHODS
+
+// Certain pointer methods aren't implemented below Rustc versions 1.26.
+// We implement a dummy version here.
+
+trait PointerMethods {
+    // Add to the pointer (use padd to avoid conflict with ptr::add).
+    unsafe fn padd(self, count: usize) -> Self;
+}
+
+impl<T> PointerMethods for *const T {
+    unsafe fn padd(self, count: usize) -> Self {
+        #[cfg(has_pointer_methods)]
+        return self.add(count);
+
+        #[cfg(not(has_pointer_methods))]
+        return self.offset(count as isize);
+    }
+}
+
+impl<T> PointerMethods for *mut T {
+    unsafe fn padd(self, count: usize) -> Self {
+        #[cfg(has_pointer_methods)]
+        return self.add(count);
+
+        #[cfg(not(has_pointer_methods))]
+        return self.offset(count as isize);
+    }
+}
+
 // ARRAY
 
 /// Types that can be used as the backing store for a StackVec
@@ -412,7 +442,7 @@ impl<A: Array> StackVec<A> {
     pub fn push(&mut self, value: A::Item) {
         assert!(self.len() < self.capacity());
         unsafe {
-            ptr::write(self.as_mut_ptr().add(self.length), value);
+            ptr::write(self.as_mut_ptr().padd(self.length), value);
             self.length += 1;
         }
     }
@@ -425,7 +455,7 @@ impl<A: Array> StackVec<A> {
                 None
             } else {
                 self.length -=1;
-                Some(ptr::read(self.as_mut_ptr().add(self.length)))
+                Some(ptr::read(self.as_mut_ptr().padd(self.length)))
             }
         }
     }
@@ -439,7 +469,7 @@ impl<A: Array> StackVec<A> {
         unsafe {
             while len < self.length {
                 self.length -= 1;
-                ptr::drop_in_place(self.as_mut_ptr().add(self.length));
+                ptr::drop_in_place(self.as_mut_ptr().padd(self.length));
             }
         }
     }
@@ -486,7 +516,7 @@ impl<A: Array> StackVec<A> {
         assert!(index < self.len());
         unsafe {
             self.length -= 1;
-            let ptr = self.as_mut_ptr().add(index);
+            let ptr = self.as_mut_ptr().padd(index);
             let item = ptr::read(ptr);
             ptr::copy(ptr.offset(1), ptr, self.length - index);
             item
@@ -499,7 +529,7 @@ impl<A: Array> StackVec<A> {
     pub fn insert(&mut self, index: usize, element: A::Item) {
         assert!(index < self.len() && self.len() < self.capacity());
         unsafe {
-            let ptr = self.as_mut_ptr().add(index);
+            let ptr = self.as_mut_ptr().padd(index);
             ptr::copy(ptr, ptr.offset(1), self.length - index);
             ptr::write(ptr, element);
             self.length += 1;
@@ -522,17 +552,17 @@ impl<A: Array> StackVec<A> {
 
         unsafe {
             let old_len = self.len();
-            let ptr = self.as_mut_ptr().add(index);
+            let ptr = self.as_mut_ptr().padd(index);
 
             // Move the trailing elements.
-            ptr::copy(ptr, ptr.add(lower_bound), old_len - index);
+            ptr::copy(ptr, ptr.padd(lower_bound), old_len - index);
 
             // In case the iterator panics, don't double-drop the items we just copied above.
             self.set_len(index);
 
             let mut num_added = 0;
             for element in iter {
-                let mut cur = ptr.add(num_added);
+                let cur = ptr.padd(num_added);
                 ptr::write(cur, element);
                 num_added += 1;
             }
@@ -647,8 +677,8 @@ impl<A: Array> StackVec<A> where A::Item: Copy {
         unsafe {
             let len = self.len();
             let slice_ptr = slice.as_ptr();
-            let ptr = self.as_mut_ptr().add(index);
-            ptr::copy(ptr, ptr.add(slice.len()), len - index);
+            let ptr = self.as_mut_ptr().padd(index);
+            ptr::copy(ptr, ptr.padd(slice.len()), len - index);
             ptr::copy_nonoverlapping(slice_ptr, ptr, slice.len());
             self.set_len(len + slice.len());
         }
@@ -814,8 +844,12 @@ impl_index!(usize, A::Item);
 impl_index!(ops::Range<usize>, [A::Item]);
 impl_index!(ops::RangeFrom<usize>, [A::Item]);
 impl_index!(ops::RangeFull, [A::Item]);
-impl_index!(ops::RangeInclusive<usize>, [A::Item]);
 impl_index!(ops::RangeTo<usize>, [A::Item]);
+
+#[cfg(has_range_inclusive)]
+impl_index!(ops::RangeInclusive<usize>, [A::Item]);
+
+#[cfg(has_range_inclusive)]
 impl_index!(ops::RangeToInclusive<usize>, [A::Item]);
 
 impl<A: Array> ExtendFromSlice<A::Item> for StackVec<A> where A::Item: Copy {
@@ -854,11 +888,11 @@ impl<A: Array> Extend<A::Item> for StackVec<A> {
 
         unsafe {
             let len = self.len();
-            let ptr = self.as_mut_ptr().add(len);
+            let ptr = self.as_mut_ptr().padd(len);
             let mut count = 0;
             while count < lower_bound {
                 if let Some(out) = iter.next() {
-                    ptr::write(ptr.add(count), out);
+                    ptr::write(ptr.padd(count), out);
                     count += 1;
                 } else {
                     break;
@@ -974,7 +1008,7 @@ impl<A: Array> Iterator for IntoIter<A> {
             unsafe {
                 let current = self.current;
                 self.current += 1;
-                Some(ptr::read(self.data.as_ptr().add(current)))
+                Some(ptr::read(self.data.as_ptr().padd(current)))
             }
         }
     }
@@ -995,7 +1029,7 @@ impl<A: Array> DoubleEndedIterator for IntoIter<A> {
         else {
             unsafe {
                 self.end -= 1;
-                Some(ptr::read(self.data.as_ptr().add(self.end)))
+                Some(ptr::read(self.data.as_ptr().padd(self.end)))
             }
         }
     }
